@@ -519,9 +519,6 @@ swfdec_as_context_dispose (GObject *object)
 
   while (context->stack)
     swfdec_as_stack_pop_segment (context);
-  /* We need to make sure there's no exception here. Otherwise collecting 
-   * frames that are inside a try block will assert */
-  swfdec_as_context_catch (context, NULL);
   swfdec_as_context_collect (context);
   if (context->memory != 0) {
     g_critical ("%zu bytes of memory left over\n", context->memory);
@@ -840,9 +837,7 @@ start:
     SwfdecAsValue rval = { 0, };
     if (frame->argc >= native->min_args && 
 	(native->type == 0 || 
-	 g_type_is_a (G_OBJECT_TYPE (frame->thisp), native->type)) &&
-	(!frame->construct || native->construct_type == 0 ||
-	 g_type_is_a (G_OBJECT_TYPE (frame->thisp), native->construct_type))) {
+	 g_type_is_a (G_OBJECT_TYPE (frame->thisp), native->type))) {
       SwfdecAsValue *argv;
       /* accumulate argv */
       if (frame->argc == 0 || frame->argv != NULL) {
@@ -873,11 +868,6 @@ start:
 	  argv, &rval);
       if (argv != frame->argv)
 	g_free (argv);
-    } else {
-      if (frame->construct && native->construct_type != 0 &&
-	 !g_type_is_a (G_OBJECT_TYPE (frame->thisp), native->construct_type)) {
-	SWFDEC_FIXME ("Ignoring call to native constructor with invalid type");
-      }
     }
     swfdec_as_frame_return (frame, &rval);
     goto start;
@@ -915,8 +905,6 @@ start:
       if (frame != context->frame)
 	goto start;
     }
-    if (context->exception)
-      continue;
 
     /* decode next action */
     action = *pc;
