@@ -1506,9 +1506,8 @@ swfdec_text_field_movie_replace_text (SwfdecTextFieldMovie *text,
     guint start_index, guint end_index, const char *str)
 {
   SwfdecFormatIndex *findex;
-  GSList *iter;
+  GSList *iter, *prev;
   gboolean first;
-  gsize len;
 
   g_return_if_fail (SWFDEC_IS_TEXT_FIELD_MOVIE (text));
   g_return_if_fail (end_index <= text->input->len);
@@ -1520,32 +1519,33 @@ swfdec_text_field_movie_replace_text (SwfdecTextFieldMovie *text,
   if (text->style_sheet_input)
     return;
 
-  len = strlen (str);
   first = TRUE;
-  iter = text->formats; 
-  while (iter) {
+  prev = NULL;
+  for (iter = text->formats; iter != NULL; iter = iter->next)
+  {
     findex = iter->data;
-    iter = iter->next;
 
-    /* remove formats of deleted text */
-    if (findex->index_ >= start_index &&
-	(end_index == text->input->len ||
-	 (iter != NULL &&
-	  ((SwfdecFormatIndex *) iter->data)->index_ <= end_index)) &&
-	text->formats->next != NULL) {
-      text->formats = g_slist_remove (text->formats, findex);
-      g_free (findex);
-      continue;
-    }
-    /* adapt indexes: remove deleted part, add to-be inserted text */
-    if (findex->index_ > end_index) {
-      findex->index_ = findex->index_ + start_index - end_index + len;
-    } else if (findex->index_ >= start_index) {
-      findex->index_ = findex->index_ + start_index - end_index;
+    if (findex->index_ >= start_index) {
+      if (end_index == text->input->len ||(iter->next != NULL &&
+	   ((SwfdecFormatIndex *)iter->next->data)->index_ <= end_index))
+      {
+	g_free (iter->data);
+	text->formats = g_slist_remove (text->formats, iter->data);
+	iter = (prev != NULL ? prev : text->formats);
+      }
+      else
+      {
+	findex->index_ += strlen (str) - (end_index - start_index);
+	if (first) {
+	  findex->index_ -= strlen (str);
+	  first = FALSE;
+	}
+      }
     }
+    prev = iter;
   }
 
-  if (end_index == text->input->len && text->input->len > 0) {
+  if (end_index == text->input->len) {
     if (SWFDEC_AS_OBJECT (text)->context->version < 8) {
       SWFDEC_FIXME ("replaceText to the end of the TextField might use wrong text format on version 7");
     }
